package tech.riemann.bidding.component;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.nutz.dao.Cnd;
import org.nutz.http.Header;
import org.nutz.http.Http;
import org.nutz.json.Json;
import org.nutz.lang.Lang;
import org.nutz.lang.util.NutMap;
import org.nutz.log.Logs;
import org.nutz.spring.boot.service.ExtService;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;

import tech.riemann.bidding.entity.Notice;
import tech.riemann.bidding.entity.Notice.Channel;
import tech.riemann.bidding.entity.ScheduledLog;
import tech.riemann.bidding.repository.NoticeRepository;
import tech.riemann.bidding.repository.ScheduledLogRepository;

/*
 * 通用方法,非请勿动!!!
 */
public abstract class NoticeCollector implements InitializingBean {
	/**
	 * 机器人地址
	 */
	public static final String ROBOT_URL = "https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key=7251bcc2-7158-4175-a5ef-bc529e432ee6";
	/**
	 * 机器人地址(三人群)
	 */
	public static final String ROBOT_URL_GX = "https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key=cc1e8fe7-3244-4ffe-80ac-b1528b202a01";
	/**
	 * 关键词
	 */
	public static final String KEY_WORDS = "人力外包、驻场、资源池、软件开发、技术服务、开发服务、人员外包、技术开发、现场、外协";
	/**
	 * 行业限定
	 */
	public static final String INDUSTRY_KEY_WORDS = "证券、基金、保险、信托、资管、银行、养老金、中国、国家、烟草、电力、电信、移动";
	/**
	 * 重点关注
	 */
	public static final String ATTENTION_KEY_WORDS = "资源池、外包";
	/**
	 * 黑名單
	 */
	public static final String BLOCKED_KEY_WORDS = "单一、竞争、中标、结果、询价、公示、终止、成交、延期、磋商、失败";

	public static final Pattern DATE_PATTERN = Pattern.compile("\\d{4}[-]\\d{2}[-]\\d{2}");
	public static final Pattern NUMBER_PATTERN = Pattern.compile("\\d+");
	/**
	 * 待发送队列
	 */
	public static final LinkedBlockingQueue<Notice> NOTICES_QUEUE = new LinkedBlockingQueue<>();

	/**
	 * 提取日期
	 * 
	 * @param info
	 * @return
	 */
	public LocalDate date(String info) {
		Matcher matcher = DATE_PATTERN.matcher(info);
		if (matcher.find()) {
			String date = matcher.group();
			return LocalDate.parse(date, DateTimeFormatter.ofPattern("yyyy-MM-dd"));
		}
		return LocalDate.now();
	}

	/**
	 * 提取数字
	 * 
	 * @param info
	 * @return
	 */
	public static long number(String info) {
		Matcher matcher = NUMBER_PATTERN.matcher(info);
		if (matcher.find()) {
			String data = matcher.group();
			return Long.parseLong(data);
		}
		return 0;
	}

	/**
	 * 采集渠道
	 * 
	 * @return
	 */
	public abstract Channel channel();

	/**
	 * 采集方法,对应渠道自行实现
	 */
	public abstract void collect();

	protected static final NutMap DUPLICATE_STATUS = NutMap.NEW();

	/**
	 * 开始采集,由采集渠道自行决定定时频率等
	 */
	@Async
	@Scheduled(cron = "0 */2 * * * ?")
	public void start() {
		DUPLICATE_STATUS.setv(channel().name(), 0);// 新一轮调度,重置当前渠道的重复数据
		Logs.get().debugf("渠道:%s 开始进行数据采集!", channel().getDescription());
		ScheduledLog log = startLog();
		collect();
		stopLog(log);
	}

	/**
	 * 推送数据
	 * 
	 * @param notices
	 * @return
	 */
	public boolean pushNotices(List<Notice> notices) {
		return notices.stream().map(this::pushNotice).allMatch(item -> item); // 全部都OK,不然就终止
	}

	/**
	 * 推送数据
	 * 
	 * @param notice
	 * @return
	 */
	public boolean pushNotice(Notice notice) {
		if (notice.getPublishDate().isBefore(LocalDate.now().minusDays(2))) { // 两天之前的数据,直接不要了
			return false;
		}
		if (noticeRepository().count(Cnd.where(Notice::getChannel, ExtService.EQ, notice.getChannel())
				.and(Notice::getKey, ExtService.EQ, notice.getKey())) == 0) {
			// 没有,插入数据,发送通知
			noticeRepository().insert(notice);
			sendMessage(notice);
			return true;
		} else {
			int currentDuplicate = DUPLICATE_STATUS.getInt(channel().name());
			if (currentDuplicate >= 20) {// 超过3次重复,终止采集
				DUPLICATE_STATUS.setv(channel().name(), 0);
				return false;
			}
			DUPLICATE_STATUS.setv(channel().name(), currentDuplicate + 1); // 重复的次数
			return true;
		}
	}

	/**
	 * 看门狗,单独线程扫描待发送渠道进行消息发送,需要渠道实现通过
	 */
	@Override
	public void afterPropertiesSet() {
		String messageTemplate = """
				<font>%s</font>
				         >发布时间:<font color='comment'>%s</font>
				         >来源:<font color='comment'>%s</font>
				         >详情:<font color='comment'>[点击查看详情](%s)</font>
				""";
		new Thread(() -> {
			while (true) {
				Notice notice = NOTICES_QUEUE.poll();
				if (notice != null) {
					// 发送消息通知
					Http.post3(ROBOT_URL,
							Json.toJson(NutMap.NEW().addv("msgtype", "markdown").addv("markdown",
									NutMap.NEW().addv("content",
											String.format(messageTemplate, notice.getTitle(), notice.getPublishDate(),
													notice.getChannel().getDescription(), notice.getUrl())))),
							Header.create().asJsonContentType(), 500);
					Http.post3(ROBOT_URL_GX,
							Json.toJson(NutMap.NEW().addv("msgtype", "markdown").addv("markdown",
									NutMap.NEW().addv("content",
											String.format(messageTemplate, notice.getTitle(), notice.getPublishDate(),
													notice.getChannel().getDescription(), notice.getUrl())))),
							Header.create().asJsonContentType(), 500);
					if (attention(notice)) {
						// 发送AT助理消息
						Http.post3(ROBOT_URL,
								Json.toJson(NutMap.NEW().addv("msgtype", "text").addv("text",
										NutMap.NEW().addv("mentioned_mobile_list", Lang.list("13811608471","13018059968","18343641000","18996359755","13673518683"))
												.addv("content",  "检测到【资源池、外包】项目，请立即确认！"+ notice.getTitle() +": "+ notice.getUrl()))),
								Header.create().asJsonContentType(), 500);
					}
				}
				try {
					Thread.sleep(notice == null ? 100 : 5000);
				} catch (InterruptedException e) {
					throw Lang.wrapThrow(e);
				}
			}
		}).start();
	}

	public boolean attention(Notice notice) {
		List<String> keyWords = Lang.list(ATTENTION_KEY_WORDS.split("、"));
		return keyWords.stream().anyMatch(key -> notice.getTitle().contains(key))
				|| keyWords.stream().anyMatch(key -> notice.getContent().contains(key));
	}

	/**
	 * 发送消息
	 * 
	 * @param notice
	 */
	public void sendMessage(Notice notice) {
		/**
		 * 1. 关键词匹配放入队列<br>
		 * 2. 队列数据单独线程定时消费
		 */
		if (match(notice)) {
			NOTICES_QUEUE.add(notice);
		}
	}

	/**
	 * 发送消息的匹配规则
	 * 
	 * @param notice 公告
	 * @return 是否匹配
	 */
	public boolean match(Notice notice) {
		List<String> blacks = Lang.list(BLOCKED_KEY_WORDS.split("、"));
		List<String> keyWords = Lang.list(KEY_WORDS.split("、"));
		if (blacks.stream().noneMatch(key -> notice.getTitle().contains(key))
				&& blacks.stream().noneMatch(key -> notice.getContent().contains(key))) {
			return keyWords.stream().anyMatch(key -> notice.getTitle().contains(key))
					|| keyWords.stream().anyMatch(key -> notice.getContent().contains(key));
		}
		return false;
	}

	public abstract ScheduledLogRepository scheduledLogRepository();

	public abstract NoticeRepository noticeRepository();

	/**
	 * 
	 * 开始日志
	 * 
	 * @return
	 */
	public ScheduledLog startLog() {
		return ScheduledLog.builder().channel(channel()).threadId(Thread.currentThread().getId()).build();
	}

	/**
	 * 停止日志
	 * 
	 * @param log
	 */
	public void stopLog(ScheduledLog log) {
		log.setEnd(LocalDateTime.now());
		scheduledLogRepository().insert(log);
	}
	public static void main(String[] args) {
		Http.post3(ROBOT_URL,
				Json.toJson(NutMap.NEW().addv("msgtype", "text").addv("text",
						NutMap.NEW().addv("mentioned_mobile_list", Lang.list("13811608471","13018059968","18343641000","18996359755","13673518683"))
								.addv("content", "本条消息是测试，检测到【资源池、人力外包】项目，请立即确认！"))),
				Header.create().asJsonContentType(), 500);
	}
}
